@@ -65,7 +65,7 @@
 # make it effectively proprietary.  To prevent this, the GPL assures that
 # patents cannot be used to render the program non-free.
 
-# Open Generation, Storage, and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 4.4.0 - March 11, 2022
+# Open Generation, Storage, and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 4.5.1 - March 25, 2022
 # simplicity and transparency in power systems planning
 
 # Developed by
@@ -89,6 +89,8 @@ import os
 # import pkg_resources
 from .openTEPES import openTEPES_run
 
+print('\n #### Academic research license - for non-commercial use only #### \n')
+
 parser = argparse.ArgumentParser(description='Introducing main parameters...')
 parser.add_argument('--case',   type=str, default=None)
 parser.add_argument('--dir',    type=str, default=None)
@@ -126,7 +128,7 @@ def main():
     print(sys.argv)
     print(args)
     openTEPES_run(args.dir, args.case, args.solver, args.log)
-    sys.exit('End of the run                         ...')
+    sys.exit('End of the run                ************')
 
 if __name__ == '__main__':
     main()
